import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/toPromise';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Injectable, NgModule, Directive, ElementRef, Input, HostListener, Output, EventEmitter, ViewContainerRef, Component, forwardRef, NgZone, ViewChild, Renderer, defineInjectable, inject } from '@angular/core';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SelectService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
        this.headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    }
    /**
     * @param {?} url
     * @param {?} parametros
     * @return {?}
     */
    getResultados(url, parametros) {
        let /** @type {?} */ urlParams = this.paramns(parametros, new HttpParams());
        return this.http.get(url, { headers: this.headers, params: urlParams });
    }
    /**
     * @param {?} a
     * @param {?} urlParams
     * @return {?}
     */
    paramns(a, urlParams) {
        for (let /** @type {?} */ prefix in a) {
            urlParams = this.buildParams(prefix, a[prefix], urlParams);
        }
        return urlParams;
    }
    /**
     * @param {?} prefix
     * @param {?} obj
     * @param {?} urlParams
     * @return {?}
     */
    buildParams(prefix, obj, urlParams) {
        if (obj instanceof Array) {
            for (let /** @type {?} */ i in obj) {
                urlParams = this.buildParams(prefix + "[" + (typeof obj[i] === "object" || Array.isArray(obj[i]) ? i : "") + "]", obj[i], urlParams);
            }
            return urlParams;
        }
        else if (obj instanceof Object) {
            for (let /** @type {?} */ name in obj) {
                if (obj[name] instanceof Date) {
                    obj[name] = JSON.stringify(obj[name]);
                }
                urlParams = this.buildParams(prefix + "[" + name + "]", obj[name], urlParams);
            }
            return urlParams;
        }
        else {
            if (obj !== null && obj !== undefined) {
                urlParams = urlParams.append(prefix, obj);
            }
            return urlParams;
        }
    }
}
SelectService.decorators = [
    { type: Injectable, args: [{
                providedIn: "root"
            },] },
];
/** @nocollapse */
SelectService.ctorParameters = () => [
    { type: HttpClient, },
];
/** @nocollapse */ SelectService.ngInjectableDef = defineInjectable({ factory: function SelectService_Factory() { return new SelectService(inject(HttpClient)); }, token: SelectService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class HiddenDirective {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    set hidden(valor) {
        if (valor) {
            this.elementRef.nativeElement.style.display = 'none';
        }
        else {
            this.elementRef.nativeElement.style.display = 'block';
        }
    }
}
HiddenDirective.decorators = [
    { type: Directive, args: [{
                selector: '[hidden]',
            },] },
];
/** @nocollapse */
HiddenDirective.ctorParameters = () => [
    { type: ElementRef, },
];
HiddenDirective.propDecorators = {
    "hidden": [{ type: Input, args: ['hidden',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class HighlightDirective {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
    }
    /**
     * @return {?}
     */
    onMouseEnter() {
        this.highlight();
    }
    /**
     * @return {?}
     */
    onMouseLeave() {
        this.highlight();
    }
    /**
     * @return {?}
     */
    highlight() {
        if (this.elementRef.nativeElement.className.indexOf(this.classe) === -1) {
            this.elementRef.nativeElement.className += ' ' + this.classe;
        }
        else {
            this.elementRef.nativeElement.classList.remove(this.classe);
        }
    }
}
HighlightDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]',
            },] },
];
/** @nocollapse */
HighlightDirective.ctorParameters = () => [
    { type: ElementRef, },
];
HighlightDirective.propDecorators = {
    "classe": [{ type: Input, args: ['highlight',] },],
    "onMouseEnter": [{ type: HostListener, args: ['mouseenter',] },],
    "onMouseLeave": [{ type: HostListener, args: ['mouseleave',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class InfiniteScrollDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this._count = 0;
        this.OnScrollMethod = new EventEmitter();
        this._element = this.element.nativeElement;
        if (!this.scrollTrigger) {
            this.scrollTrigger = 1;
        }
    }
    /**
     * @return {?}
     */
    onScroll() {
        this._count++;
        if (this._element.scrollTop + this._element.clientHeight >= this._element.scrollHeight) {
            this.OnScrollMethod.emit(null);
        }
        else {
            if (this._element.scrollTop + this._element.clientHeight >= (this._element.scrollHeight - (this._element.scrollHeight * this.scrollTrigger))) {
                this.OnScrollMethod.emit(null);
            }
        }
    }
}
InfiniteScrollDirective.decorators = [
    { type: Directive, args: [{
                selector: '[infinite-scroll]',
                host: {
                    '(scroll)': 'onScroll($event)'
                }
            },] },
];
/** @nocollapse */
InfiniteScrollDirective.ctorParameters = () => [
    { type: ElementRef, },
];
InfiniteScrollDirective.propDecorators = {
    "scrollTrigger": [{ type: Input, args: ['scroll-distance',] },],
    "OnScrollMethod": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class InnerTemplateDirective {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    set item(item) {
        this._item = item;
        if (this.template) {
            this.template.destroy();
            this.criaTemplate();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.criaTemplate();
    }
    /**
     * @return {?}
     */
    criaTemplate() {
        if (this.templateRef) {
            this.template = this.viewContainer.createEmbeddedView(this.templateRef, {
                '\$implicit': this._item,
                'index': this.index
            });
        }
    }
}
InnerTemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[inner-template]'
            },] },
];
/** @nocollapse */
InnerTemplateDirective.ctorParameters = () => [
    { type: ViewContainerRef, },
];
InnerTemplateDirective.propDecorators = {
    "item": [{ type: Input },],
    "index": [{ type: Input },],
    "templateRef": [{ type: Input, args: ['inner-template',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Select {
    /**
     * @param {?} elementRef
     * @param {?} zone
     */
    constructor(elementRef, zone) {
        this.elementRef = elementRef;
        this.zone = zone;
        this.indiceId = 'id';
        this.indiceNome = 'nome';
        this.minimoCaracteres = 0;
        this.exibirMensagemCaracteresMinimo = true;
        this.aberto = false;
        this.focus = false;
        this.disabled = false;
        this.valoresExibir = [];
        this.change = new EventEmitter();
        this.onSelecionarItem = new EventEmitter();
        this.onBuscar = new EventEmitter();
        this.onApagar = new EventEmitter();
        this.onRemoverItem = new EventEmitter();
        this.onAbrir = new EventEmitter();
        this.onFechar = new EventEmitter();
        this.onLimpar = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.minimoCaracteres == 0) {
            this.exibirMensagemCaracteresMinimo = false;
        }
        this.elementRef.nativeElement.setAttribute("style", "float:left;width:100%");
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    setFocus(valor) {
        if (this.disabled) {
            return false;
        }
        this.focus = valor;
    }
    /**
     * @return {?}
     */
    fechar() {
        this.aberto = false;
        this.valoresExibir = [];
        this.setFocus(true);
        this.setFocus(false);
        this.onFechar.emit(true);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    selecionar(item) {
        this.updateValue(this.data);
        this.onSelecionarItem.emit(item);
        //apaga campo de busca
        this.campoBusca.nativeElement.value = '';
        this.fechar();
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    keyup(valor) {
        if (this.valorPesquisado == valor) {
            return null;
        }
        this.valoresExibir = [];
        this.valorPesquisado = valor;
        this.buscar();
    }
    /**
     * @return {?}
     */
    buscar() {
    }
    /**
     * @return {?}
     */
    abrir() {
    }
    /**
     * quando está marcado para exibir resultado completo
     * @param {?} item
     * @param {?} event
     * @return {?}
     */
    remove(item, event) {
        event.stopPropagation();
        this.fechar();
        // let itens
        let /** @type {?} */ posicao = this.data.findIndex(procurar => procurar[this.indiceId] == item[this.indiceId]);
        if (posicao !== -1) {
            this.data.splice(posicao, 1);
        }
        this.onRemoverItem.emit(item);
        if (this.data.length == 0) {
            this.aberto = false;
            this.abrir();
        }
    }
    /**
     * @return {?}
     */
    getClassGeral() {
        return {
            'select2-container--focus': this.focus,
            'select2 select2-container select2-container--bootstrap select2-container--above': true,
            'select2-container--disabled': this.disabled
        };
    }
    /**
     * @param {?} event
     * @return {?}
     */
    clickForaComponent(event) {
        if (!this.elementRef.nativeElement.contains(event.target) && this.aberto) {
            this.fechar();
        }
    }
    /**
     * @return {?}
     */
    validaCaracteresMinimo() {
        if (this.minimoCaracteres > 0) {
            if (this.valorPesquisado.length < this.minimoCaracteres) {
                return false;
            }
            this.exibirMensagemCaracteresMinimo = false;
        }
        return true;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.data = value;
        this._value = value;
        this.updateValue(this._value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.zone.run(() => {
            if (this._value != value) {
                this._value = value;
                this.onChange(value);
                this._onTouchedCallback();
                // this.change.emit(value);
            }
        });
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onChange(_) { }
    /**
     * @return {?}
     */
    onTouched() { }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) { this.onChange = fn; }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) { this.onTouched = fn; }
    /**
     * @param {?} _
     * @return {?}
     */
    _onChangeCallback(_) { }
    /**
     * @return {?}
     */
    _onTouchedCallback() { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MultipleComponent extends Select {
    /**
     * @param {?} elementRef
     * @param {?} zone
     */
    constructor(elementRef, zone) {
        super(elementRef, zone);
        this.elementRef = elementRef;
        this.zone = zone;
        this.name = '';
        this.classe = '';
        this.placeholder = 'Selecione';
        this.minimoCaracteres = 0;
        this.disabled = false;
        this.indiceId = 'id';
        this.indiceNome = 'nome';
        this.change = new EventEmitter();
        this.onSelecionarItem = new EventEmitter();
        this.onBuscar = new EventEmitter();
        this.onApagar = new EventEmitter();
        this.onRemoverItem = new EventEmitter();
        this.onAbrir = new EventEmitter();
        this.onFechar = new EventEmitter();
        this.onLimpar = new EventEmitter();
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    set valores(valor) {
        this._valores = valor;
        this.valoresExibir = valor;
    }
    /**
     * @return {?}
     */
    abrir() {
        if (this.disabled) {
            return false;
        }
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.aberto = true;
            if (!this.data) {
                this.data = [];
            }
            if (!this._value) {
                this._value = [];
            }
            this.valorPesquisado = '';
            this.campoBusca.nativeElement.focus();
            this.setFocus(true);
            this.onAbrir.emit(true);
            this.buscar();
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    selecionar(item) {
        //verifica se ja foi adicionado
        let /** @type {?} */ jaExiste = this.data.findIndex(procurar => procurar[this.indiceId] == item[this.indiceId]);
        if (jaExiste === -1) {
            this.data.push(item);
        }
        this.updateValue(this.data);
        this.onSelecionarItem.emit(item);
        this.fechar();
        //apaga campo de busca
        this.campoBusca.nativeElement.value = '';
    }
    /**
     * @return {?}
     */
    getSomenteId() {
        let /** @type {?} */ ids = [];
        for (let /** @type {?} */ item of this.data) {
            ids.push(item[this.indiceId]);
        }
        return ids;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    limpar(event) {
        event.stopPropagation();
        this.data = null;
        this.updateValue(null);
        this.onLimpar.emit(true);
        this.aberto = false;
        this.abrir();
    }
    /**
     * @param {?} item
     * @param {?} event
     * @return {?}
     */
    remove(item, event) {
        super.remove(item, event);
        this.updateValue(this.data);
    }
    /**
     * @return {?}
     */
    buscar() {
        if (this.validaCaracteresMinimo() == false) {
            return false;
        }
        this.onBuscar.emit(this.valorPesquisado);
        this.valoresExibir = this._valores.filter(item => {
            let /** @type {?} */ pesquisa = item[this.indiceNome].toString().toLocaleLowerCase().indexOf(this.valorPesquisado.toString().toLocaleLowerCase());
            //verifica se o item da lista original ja n esta add na lista data
            let /** @type {?} */ jaFoiAdicionado = this.data.findIndex(procurar => procurar[this.indiceId] == item[this.indiceId]);
            if (pesquisa !== -1 && jaFoiAdicionado === -1) {
                return item;
            }
        });
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    ;
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = (v);
            this._onChangeCallback(v);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.data = value;
        this._value = value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.zone.run(() => {
            this._value = value;
            this.onChange(value);
            this._onTouchedCallback();
            this.change.emit(value);
        });
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onChange(_) { }
    /**
     * @return {?}
     */
    onTouched() { }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) { this.onChange = fn; }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) { this.onTouched = fn; }
    /**
     * @param {?} _
     * @return {?}
     */
    _onChangeCallback(_) { }
    /**
     * @return {?}
     */
    _onTouchedCallback() { }
}
MultipleComponent.decorators = [
    { type: Component, args: [{
                selector: 'select2-multiple',
                template: `
        <!--select simples-->
        <span [ngClass]="getClassGeral()" dir="ltr" (click)="abrir()">
        <span class="selection">
            <span class="select2-selection select2-selection--multiple" role="combobox" aria-haspopup="true" aria-expanded="false" tabindex="-1">
            <ul class="select2-selection__rendered">
                <li [hidden]="data" [innerHtml]="placeholder"  class="select2-selection__placeholder"></li>
                <span [hidden]="!data" >
                    <span [inner-template]="templateSelecionado || templateSelecionadoInterno" [item]="this"></span>
                </span>
                <li class="select2-selection__arrow" role="presentation"><b role="presentation"></b></li>
            </ul>
        </span>
        </span>
        <span class="dropdown-wrapper" aria-hidden="true"></span>
    </span>

        <!--quando clicar campo-->
        <div [hidden]="!aberto">
        <span class="select2-container select2-container--bootstrap select2-container--open">
            <span class="select2-dropdown select2-dropdown--below" dir="ltr" >
                <span class="select2-search select2-search--dropdown">
                    <input (keyup)="keyup($event.target.value)" #campoBusca placeholder="Digite algo" autofocus class="select2-search__field" type="search" tabindex="0" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" role="textbox">
                </span>
                <span class="select2-results">
                    <ul class="select2-results__options" role="tree"  aria-expanded="true" aria-hidden="false">
                        <span [hidden]="exibirMensagemCaracteresMinimo">
                            <span [hidden]="valoresExibir.length == 0" >
                                <li *ngFor="let item of valoresExibir" (click)="selecionar(item)" class="select2-results__option" highlight="select2-results__option--highlighted" role="treeitem" [attr.aria-selected]="((_value) && (item[indiceId] == _value[indiceId])) ? true : false">
                                    <span  [inner-template]="templateResultado || templateResultadoInterno" [item]="item"></span>
                                </li>
                            </span>
                            <li [hidden]="valoresExibir.length > 0" class="select2-results__option select2-results__message" aria-live="assertive">
                                <span  [inner-template]="templateSemResultado || templateSemResultadoInterno" [item]="{pesquisa:valorPesquisado}"></span>
                            </li>
                        </span>
                        <li [hidden]="exibirMensagemCaracteresMinimo == false" class="select2-results__option select2-results__message">
                            Digite {{minimoCaracteres}} ou mais caracteres para realizar a busca
                        </li>
                    </ul>
                </span>
            </span>
        </span>
        </div>



        <ng-template #templateSemResultadoInterno>
            Nenhum resultado encontrado
        </ng-template>
        <ng-template #templateResultadoInterno let-valor>
            {{valor[indiceNome]}}
        </ng-template>
        <ng-template #templateSelecionadoInterno let-valor>
        <span *ngIf="valor">
            <li *ngFor="let item of valor.data" class="select2-selection__choice">
                <span class="select2-selection__choice__remove" role="presentation" (click)="valor.remove(item, $event)">×</span>
                {{item[indiceNome]}}
            </li>
        </span>
        </ng-template>
    `,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => MultipleComponent),
                        multi: true
                    }
                ],
                host: {
                    '(document:click)': 'clickForaComponent($event)',
                    '(focus)': 'setFocus(true)',
                    '(blur)': 'setFocus(false)',
                },
            },] },
];
/** @nocollapse */
MultipleComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
];
MultipleComponent.propDecorators = {
    "name": [{ type: Input },],
    "classe": [{ type: Input },],
    "placeholder": [{ type: Input },],
    "minimoCaracteres": [{ type: Input },],
    "templateResultado": [{ type: Input },],
    "templateSelecionado": [{ type: Input },],
    "templateSemResultado": [{ type: Input },],
    "disabled": [{ type: Input },],
    "indiceId": [{ type: Input },],
    "indiceNome": [{ type: Input },],
    "valores": [{ type: Input },],
    "change": [{ type: Output },],
    "onSelecionarItem": [{ type: Output },],
    "onBuscar": [{ type: Output },],
    "onApagar": [{ type: Output },],
    "onRemoverItem": [{ type: Output },],
    "onAbrir": [{ type: Output },],
    "onFechar": [{ type: Output },],
    "onLimpar": [{ type: Output },],
    "campoBusca": [{ type: ViewChild, args: ['campoBusca',] },],
    "value": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RequestComponent extends Select {
    /**
     * @param {?} elementRef
     * @param {?} zone
     * @param {?} requisicao
     * @param {?} renderer
     */
    constructor(elementRef, zone, requisicao, renderer) {
        super(elementRef, zone);
        this.elementRef = elementRef;
        this.zone = zone;
        this.requisicao = requisicao;
        this.renderer = renderer;
        this.name = '';
        this.classe = '';
        this.minimoCaracteres = 0;
        this.placeholder = 'Selecione';
        this.disabled = false;
        this.indiceId = 'id';
        this.indiceNome = 'nome';
        this.change = new EventEmitter();
        this.onSelecionarItem = new EventEmitter();
        this.onBuscar = new EventEmitter();
        this.onApagar = new EventEmitter();
        this.onRemoverItem = new EventEmitter();
        this.onAbrir = new EventEmitter();
        this.onFechar = new EventEmitter();
        this.onLimpar = new EventEmitter();
        this.onErro = new EventEmitter();
        this.onProcessaResultado = new EventEmitter();
        this.onProcessaParametros = new EventEmitter();
        this.pagina = 1;
        this.quantidadePadrao = 0;
        this.buscando = false;
        this.semResultado = false;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscrebeBusca) {
            this.subscrebeBusca.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    abrir() {
        if (this.disabled) {
            return false;
        }
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.setFocus(true);
            this.aberto = true;
            // this.renderer.invokeElementMethod(this.campoBusca.nativeElement, 'focus', []);
            this.pagina = 1;
            this.semResultado = false;
            this.quantidadePadrao = 0;
            this.valorPesquisado = '';
            this.campoBusca.nativeElement.value = '';
            this.setFocus(true);
            this.onAbrir.emit(true);
            this.buscar();
        }
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    keyup(valor) {
        if (this.valorPesquisado == valor) {
            return null;
        }
        this.valoresExibir = [];
        this.valorPesquisado = valor;
        this.pagina = 1;
        this.buscar();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    selecionar(item) {
        this.data = item;
        super.selecionar(item);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    limpar(event) {
        event.stopPropagation();
        this.data = null;
        this.updateValue(null);
        this.onLimpar.emit(true);
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.abrir();
        }
    }
    /**
     * @return {?}
     */
    buscar() {
        if (this.validaCaracteresMinimo() == false) {
            return false;
        }
        this.buscando = true;
        // this.valoresExibir = this._valores.filter(item => item[this.indiceNome].indexOf(this.valorPesquisado) !== -1);
        let /** @type {?} */ parametros = {
            enviado: {
                pagina: this.pagina,
                valorPesquisado: this.valorPesquisado,
            },
            retorno: {}
        };
        //mata o subscribe
        this.ngOnDestroy();
        this.onBuscar.emit(this.valorPesquisado);
        this.onProcessaParametros.emit(parametros);
        this.subscrebeBusca = this.requisicao.getResultados(this.url, parametros.retorno).subscribe(resultado => {
            let /** @type {?} */ objEmit = { enviado: resultado, retorno: [] };
            this.onProcessaResultado.emit(objEmit);
            let /** @type {?} */ exibirResultado = objEmit.retorno;
            if (Array.isArray(exibirResultado)) {
                if (this.quantidadePadrao == 0) {
                    this.quantidadePadrao = exibirResultado.length;
                }
                if (this.quantidadePadrao != exibirResultado.length) {
                    this.semResultado = true;
                }
                this.valoresExibir = this.valoresExibir.concat(exibirResultado);
            }
            else {
                console.log('Ng2Select2Bootstrap: retorno não é um select', exibirResultado);
            }
            this.buscando = false;
        }, erro => {
            console.log(erro);
            if (erro.status == 409) {
                this.semResultado = true;
            }
            this.onErro.emit(erro);
            this.buscando = false;
        });
    }
    /**
     * @return {?}
     */
    onScroll() {
        if (!this.buscando && !this.semResultado) {
            this.pagina++;
            this.buscar();
        }
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    ;
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = v;
            this._onChangeCallback(v);
        }
    }
}
RequestComponent.decorators = [
    { type: Component, args: [{
                selector: 'select2-request',
                template: `

<!--select simples-->
<span [ngClass]="getClassGeral()" [tabindex]="tabIndex" dir="ltr" (focus)="abrir()" (click)="abrir()">
        <span class="selection">
            <!--simples-->
            <span class="select2-selection select2-selection--single" role="combobox" aria-haspopup="true" aria-expanded="false" tabindex="0" [ngStyle]="styleItemSelecionado">
                <span class="select2-selection__rendered">
                    <span [hidden]="!data" (click)="limpar($event)" class="select2-selection__clear">×</span>
                    <span [hidden]="!data" [inner-template]="templateSelecionado || templateSelecionadoInterno" [item]="data"></span>
                    <span [hidden]="data" [innerHtml]="placeholder" class="select2-selection__placeholder"></span>

                </span>
                <span class="select2-selection__arrow" role="presentation">
                    <b role="presentation"></b>
                </span>
            </span>
        </span>
        <span class="dropdown-wrapper" aria-hidden="true"></span>
    </span>

<!--quando clicar campo-->
<div [hidden]="!aberto">
    <div class="select2-container select2-container--bootstrap select2-container--open">
        <div class="select2-dropdown select2-dropdown--below" style="z-index: 999999" dir="ltr" >
            <div class="select2-search select2-search--dropdown">
                <input (keyup)="keyup($event.target.value)" #campoBusca placeholder="Digite algo" autofocus class="select2-search__field" type="search" tabindex="0" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" role="textbox">
            </div>
            <span class="select2-results" aria-expanded="true" aria-hidden="false">
                        <ul
                                class="select2-results__options"
                                role="tree"
                                infinite-scroll
                                scroll-distance="0,30"
                                (OnScrollMethod)="onScroll()"
                                style=" overflow: auto!important;"
                        >
                            <div [style.display]="(exibirMensagemCaracteresMinimo) ? 'none': ''">
                                <div [style.display]="(valoresExibir.length == 0) ? 'none':''" >
                                    <!--<ng-template -->
                                    <!--*ngIf="templateLista"-->
                                    <!--[inner-template]="templateLista" -->
                                    <!--[item]="{valoresExibir: valoresExibir, component: this}"-->
                                    <!--&gt;-->
                                    <!--</ng-template>-->
                                    <!--<ng-template *ngIf="!templateLista">-->

                                    <!--</ng-template>-->
                                    <ng-template [inner-template]="templateLista || templateListaInterno" [item]="{component: this}">

                                    </ng-template>

                                </div>

                                <li [style.display]="(buscando || valoresExibir.length > 0) ? 'none' : ''" class="select2-results__option select2-results__message" aria-live="assertive">
                                    <ng-template  [inner-template]="templateSemResultado || templateSemResultadoInterno" [item]="{pesquisa:valorPesquisado}"></ng-template>
                                </li>
                                <li [style.display]="(!buscando) ? 'none' : ''">
                                    <ng-template [inner-template]="templateBuscando || templateBuscandoInterno"></ng-template>
                                </li>
                            </div>
                            <li [style.display]="(exibirMensagemCaracteresMinimo == false) ? 'none': ''" class="select2-results__option select2-results__message">
                                Digite {{minimoCaracteres}} ou mais caracteres para realizar a busca
                            </li>
                        </ul>
                    </span>
        </div>
    </div>
</div>



<ng-template #templateSemResultadoInterno>
    Nenhum resultado encontrado
</ng-template>
<ng-template #templateResultadoInterno let-valor>
    {{valor[indiceNome]}}
</ng-template>
<ng-template #templateSelecionadoInterno let-valor>
        <span *ngIf="valor"  [ngStyle]="styleItemSelecionado">
            {{valor[indiceNome]}}
        </span>
</ng-template>
<ng-template #templateBuscandoInterno>
    <span class="fa fa-5x fa-refresh fa-spin" style=" width: auto;margin:auto 50% !important"></span>
</ng-template>
<ng-template #templateListaInterno let-valor>
    <li
            *ngFor="let item of valor.component.valoresExibir"
            (click)="valor.component.selecionar(item)" class="select2-results__option"
            highlight="select2-results__option--highlighted"
            role="treeitem"
            [attr.aria-selected]="((_value) && (item[valor.component.indiceId] == _value[valor.component.indiceId])) ? true : false"

    >
        <div [inner-template]="templateResultado || templateResultadoInterno" [item]="item"></div>
    </li>
</ng-template>`,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => RequestComponent),
                        multi: true
                    }
                ],
                host: {
                    '(document:click)': 'clickForaComponent($event)',
                    '(focus)': 'abrir()',
                    '(blur)': 'fechar()',
                },
            },] },
];
/** @nocollapse */
RequestComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
    { type: SelectService, },
    { type: Renderer, },
];
RequestComponent.propDecorators = {
    "name": [{ type: Input },],
    "classe": [{ type: Input },],
    "tabIndex": [{ type: Input },],
    "styleItemSelecionado": [{ type: Input },],
    "minimoCaracteres": [{ type: Input },],
    "placeholder": [{ type: Input },],
    "templateResultado": [{ type: Input },],
    "templateLista": [{ type: Input },],
    "templateSelecionado": [{ type: Input },],
    "templateSemResultado": [{ type: Input },],
    "templateBuscando": [{ type: Input },],
    "disabled": [{ type: Input },],
    "indiceId": [{ type: Input },],
    "indiceNome": [{ type: Input },],
    "url": [{ type: Input },],
    "change": [{ type: Output },],
    "onSelecionarItem": [{ type: Output },],
    "onBuscar": [{ type: Output },],
    "onApagar": [{ type: Output },],
    "onRemoverItem": [{ type: Output },],
    "onAbrir": [{ type: Output },],
    "onFechar": [{ type: Output },],
    "onLimpar": [{ type: Output },],
    "onErro": [{ type: Output },],
    "onProcessaResultado": [{ type: Output },],
    "onProcessaParametros": [{ type: Output },],
    "campoBusca": [{ type: ViewChild, args: ['campoBusca',] },],
    "value": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RequestMultipleComponent extends Select {
    /**
     * @param {?} elementRef
     * @param {?} zone
     * @param {?} requisicao
     * @param {?} renderer
     */
    constructor(elementRef, zone, requisicao, renderer) {
        super(elementRef, zone);
        this.elementRef = elementRef;
        this.zone = zone;
        this.requisicao = requisicao;
        this.renderer = renderer;
        this.name = '';
        this.classe = '';
        this.placeholder = 'Selecione';
        this.minimoCaracteres = 0;
        this.disabled = false;
        this.indiceId = 'id';
        this.indiceNome = 'nome';
        this.change = new EventEmitter();
        this.onSelecionarItem = new EventEmitter();
        this.onBuscar = new EventEmitter();
        this.onApagar = new EventEmitter();
        this.onRemoverItem = new EventEmitter();
        this.onAbrir = new EventEmitter();
        this.onFechar = new EventEmitter();
        this.onLimpar = new EventEmitter();
        this.onErro = new EventEmitter();
        this.onProcessaResultado = new EventEmitter();
        this.onProcessaParametros = new EventEmitter();
        this.pagina = 1;
        this.quantidadePadrao = 0;
        this.buscando = false;
        this.semResultado = false;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscrebeBusca) {
            this.subscrebeBusca.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    abrir() {
        if (this.disabled) {
            return false;
        }
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.aberto = true;
            if (!this.data) {
                this.data = [];
            }
            this.onAbrir.emit(true);
            // this.renderer.invokeElementMethod(this.campoBusca.nativeElement, 'focus', []);
            this.pagina = 1;
            this.semResultado = false;
            this.quantidadePadrao = 0;
            this.valorPesquisado = '';
            this.campoBusca.nativeElement.value = '';
            this.setFocus(true);
            this.buscar();
        }
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    keyup(valor) {
        if (this.valorPesquisado == valor) {
            return null;
        }
        this.valoresExibir = [];
        this.valorPesquisado = valor;
        this.pagina = 1;
        this.buscar();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    selecionar(item) {
        let /** @type {?} */ jaExiste = this.data.findIndex(procurar => procurar[this.indiceId] == item[this.indiceId]);
        if (jaExiste === -1) {
            this.data.push(item);
        }
        this.updateValue(this.data);
        this.onSelecionarItem.emit(item);
        this.fechar();
        //apaga campo de busca
        this.campoBusca.nativeElement.value = '';
    }
    /**
     * @return {?}
     */
    getSomenteId() {
        let /** @type {?} */ ids = [];
        for (let /** @type {?} */ item of this.data) {
            ids.push(item[this.indiceId]);
        }
        return ids;
    }
    /**
     * @return {?}
     */
    limpar() {
        this.data = [];
        this.updateValue(null);
        this.aberto = false;
        this.onLimpar.emit(true);
        this.abrir();
    }
    /**
     * @return {?}
     */
    buscar() {
        if (this.validaCaracteresMinimo() == false) {
            return false;
        }
        this.buscando = true;
        // this.valoresExibir = this._valores.filter(item => item[this.indiceNome].indexOf(this.valorPesquisado) !== -1);
        let /** @type {?} */ parametros = {
            enviado: {
                pagina: this.pagina,
                valorPesquisado: this.valorPesquisado,
                idSelecionados: this.getSomenteId(),
            },
            retorno: {}
        };
        //mata o subscribe
        this.ngOnDestroy();
        this.onBuscar.emit(this.valorPesquisado);
        this.onProcessaParametros.emit(parametros);
        this.subscrebeBusca = this.requisicao.getResultados(this.url, parametros.retorno).subscribe(resultado => {
            let /** @type {?} */ objEmit = { enviado: resultado, retorno: [] };
            this.onProcessaResultado.emit(objEmit);
            let /** @type {?} */ exibirResultado = objEmit.retorno;
            if (Array.isArray(exibirResultado)) {
                if (this.quantidadePadrao == 0 && exibirResultado.length < 25) {
                    this.semResultado = true;
                }
                if (this.quantidadePadrao == 0) {
                    this.quantidadePadrao = exibirResultado.length;
                }
                if (this.quantidadePadrao != exibirResultado.length) {
                    this.semResultado = true;
                }
                this.valoresExibir = this.valoresExibir.concat(exibirResultado);
            }
            else {
                console.log('Ng2Select2Bootstrap: retorno não é um select', exibirResultado);
            }
            this.buscando = false;
        }, erro => {
            if (erro.status == 409) {
                this.semResultado = true;
            }
            this.onErro.emit(erro);
            this.buscando = false;
        });
    }
    /**
     * @return {?}
     */
    onScroll() {
        if (!this.buscando && !this.semResultado) {
            this.pagina++;
            this.buscar();
        }
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    ;
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = v;
            this._onChangeCallback(v);
        }
    }
}
RequestMultipleComponent.decorators = [
    { type: Component, args: [{
                selector: 'select2-request-multiple',
                template: `
        <span [ngClass]="getClassGeral()"  dir="ltr" (click)="abrir()">
        <span class="selection">
            <span class="select2-selection select2-selection--multiple" role="combobox" aria-haspopup="true" aria-expanded="false" tabindex="-1">
            <ul class="select2-selection__rendered">
                <li [hidden]="data" [innerHtml]="placeholder"  class="select2-selection__placeholder"></li>
                <span [hidden]="!data" >
                    <span [inner-template]="templateSelecionado || templateSelecionadoInterno" [item]="this"></span>
                </span>
                <li class="select2-selection__arrow" role="presentation"><b role="presentation"></b></li>
            </ul>
        </span>
        </span>
        <span class="dropdown-wrapper" aria-hidden="true"></span>
    </span>

        <!--quando clicar campo-->
        <div [hidden]="!aberto">
        <span class="select2-container select2-container--bootstrap select2-container--open">
            <span class="select2-dropdown select2-dropdown--below" style="z-index: 999999" dir="ltr" >
                <span class="select2-search select2-search--dropdown">
                    <input (keyup)="keyup($event.target.value)" #campoBusca placeholder="Digite algo" autofocus class="select2-search__field" type="search" tabindex="0" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" role="textbox">
                </span>
                <span class="select2-results" aria-expanded="true" aria-hidden="false">
                    <ul
                            class="select2-results__options"
                            role="tree"
                            infinite-scroll
                            scroll-distance="0,30"
                            (OnScrollMethod)="onScroll()"
                            style=" overflow: auto!important;"
                    >
                        <div [style.display]="(exibirMensagemCaracteresMinimo) ? 'none': ''">
                            <span [style.display]="(valoresExibir.length == 0) ? 'none':''" >
                                <ng-template [inner-template]="templateLista || templateListaInterno" [item]="{component: this}"></ng-template>
                            </span>
                            <li [style.display]="(buscando || valoresExibir.length > 0) ? 'none' : ''" class="select2-results__option select2-results__message" aria-live="assertive">
                                <span  [inner-template]="templateSemResultado || templateSemResultadoInterno" [item]="{pesquisa:valorPesquisado}"></span>
                            </li>
                            <li [style.display]="(!buscando) ? 'none' : ''">
                                <span [inner-template]="templateBuscando || templateBuscandoInterno"></span>
                            </li>
                        </div>
                        <li [style.display]="(exibirMensagemCaracteresMinimo == false) ? 'none': ''" class="select2-results__option select2-results__message">
                            Digite {{minimoCaracteres}} ou mais caracteres para realizar a busca
                        </li>
                    </ul>
                </span>
            </span>
        </span>
        </div>



        <ng-template #templateSemResultadoInterno>
            Nenhum resultado encontrado
        </ng-template>
        <ng-template #templateResultadoInterno let-valor>
            {{valor[indiceNome]}}
        </ng-template>
        <ng-template #templateSelecionadoInterno let-valor>
       <span *ngIf="valor">
            <li *ngFor="let item of valor.data" class="select2-selection__choice">
                <span class="select2-selection__choice__remove" role="presentation" (click)="valor.remove(item, $event)">×</span>
                {{item[indiceNome]}}
            </li>
        </span>
        </ng-template>
        <ng-template #templateBuscandoInterno>
            <span class="fa fa-5x fa-refresh fa-spin" style=" width: auto;margin:auto 50% !important"></span>
        </ng-template>
         <ng-template #templateListaInterno let-valor>
            <li 
                *ngFor="let item of valor.component.valoresExibir" 
                (click)="valor.component.selecionar(item)" class="select2-results__option" 
                highlight="select2-results__option--highlighted" 
                role="treeitem" 
                [attr.aria-selected]="((_value) && (item[valor.component.indiceId] == _value[valor.component.indiceId])) ? true : false"
                
            >
                <div [inner-template]="templateResultado || templateResultadoInterno" [item]="item"></div>
            </li>
        </ng-template>

    `,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => RequestMultipleComponent),
                        multi: true
                    }
                ],
                host: {
                    '(document:click)': 'clickForaComponent($event)',
                    '(focus)': 'setFocus(true)',
                    '(blur)': 'setFocus(false)',
                },
            },] },
];
/** @nocollapse */
RequestMultipleComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
    { type: SelectService, },
    { type: Renderer, },
];
RequestMultipleComponent.propDecorators = {
    "name": [{ type: Input },],
    "classe": [{ type: Input },],
    "placeholder": [{ type: Input },],
    "minimoCaracteres": [{ type: Input },],
    "templateResultado": [{ type: Input },],
    "templateLista": [{ type: Input },],
    "templateSelecionado": [{ type: Input },],
    "templateSemResultado": [{ type: Input },],
    "templateBuscando": [{ type: Input },],
    "disabled": [{ type: Input },],
    "indiceId": [{ type: Input },],
    "indiceNome": [{ type: Input },],
    "url": [{ type: Input },],
    "change": [{ type: Output },],
    "onSelecionarItem": [{ type: Output },],
    "onBuscar": [{ type: Output },],
    "onApagar": [{ type: Output },],
    "onRemoverItem": [{ type: Output },],
    "onAbrir": [{ type: Output },],
    "onFechar": [{ type: Output },],
    "onLimpar": [{ type: Output },],
    "onErro": [{ type: Output },],
    "onProcessaResultado": [{ type: Output },],
    "onProcessaParametros": [{ type: Output },],
    "campoBusca": [{ type: ViewChild, args: ['campoBusca',] },],
    "value": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SelectComponent extends Select {
    /**
     * @param {?} elementRef
     * @param {?} zone
     */
    constructor(elementRef, zone) {
        super(elementRef, zone);
        this.elementRef = elementRef;
        this.zone = zone;
        this._valores = [];
        this.name = '';
        this.classe = '';
        this.placeholder = 'Selecione';
        this.minimoCaracteres = 0;
        this.indiceId = 'id';
        this.indiceNome = 'nome';
        this.disabled = false;
        // @Output() change            = new EventEmitter<any>();
        this.onSelecionarItem = new EventEmitter();
        this.onBuscar = new EventEmitter();
        this.onApagar = new EventEmitter();
        this.onRemoverItem = new EventEmitter();
        this.onAbrir = new EventEmitter();
        this.onFechar = new EventEmitter();
        this.onLimpar = new EventEmitter();
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    set valores(valor) {
        this._valores = valor;
        this.valoresExibir = valor;
    }
    /**
     * @return {?}
     */
    abrir() {
        if (this.disabled) {
            return false;
        }
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.aberto = true;
            this.campoBusca.nativeElement.focus();
            this.valorPesquisado = '';
            this.campoBusca.nativeElement.value = '';
            this.setFocus(true);
            this.onAbrir.emit(true);
            this.buscar();
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    selecionar(item) {
        this.data = item;
        super.selecionar(item);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    limpar(event) {
        event.stopPropagation();
        this.data = null;
        this.updateValue(null);
        this.onLimpar.emit(true);
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.abrir();
        }
    }
    /**
     * @return {?}
     */
    buscar() {
        if (this.validaCaracteresMinimo() == false) {
            return false;
        }
        this.onBuscar.emit(this.valorPesquisado);
        this.valoresExibir = this._valores.filter(item => item[this.indiceNome].toString().toLocaleLowerCase().indexOf(this.valorPesquisado.toString().toLocaleLowerCase()) !== -1);
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    ;
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = v;
            this._onChangeCallback(v);
        }
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'select2-select',
                template: `
        <!--select simples-->
        <span [ngClass]="getClassGeral()" dir="ltr" (click)="abrir()">
        <span class="selection">
            <!--simples-->
            <span class="select2-selection select2-selection--single" role="combobox" aria-haspopup="true" aria-expanded="false" tabindex="0">
                <span class="select2-selection__rendered">
                    <span [hidden]="!data" (click)="limpar($event)" class="select2-selection__clear">×</span>
                    <span [hidden]="!data" class="nao-fechar" [inner-template]="templateSelecionado || templateSelecionadoInterno" [item]="data"></span>
                    <span [hidden]="data" [innerHtml]="placeholder" class="select2-selection__placeholder"></span>

                </span>
                <span class="select2-selection__arrow" role="presentation">
                    <b role="presentation"></b>
                </span>
            </span>
        </span>
        <span class="dropdown-wrapper" aria-hidden="true"></span>
    </span>

        <!--quando clicar campo-->
        <div [hidden]="!aberto">
        <span class="select2-container select2-container--bootstrap select2-container--open">
            <span class="select2-dropdown select2-dropdown--below" dir="ltr" >
                <span class="select2-search select2-search--dropdown">
                    <input (keyup)="keyup($event.target.value)" #campoBusca placeholder="Digite algo" autofocus class="select2-search__field" type="search" tabindex="0" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" role="textbox">
                </span>
                <span class="select2-results">
                    <ul class="select2-results__options" role="tree"  aria-expanded="true" aria-hidden="false">
                        <span [style.display]="(exibirMensagemCaracteresMinimo) ? 'none': ''">
                            <span [hidden]="!valoresExibir.length" >
                                <li *ngFor="let item of valoresExibir" (click)="selecionar(item)" class="select2-results__option" highlight="select2-results__option--highlighted" role="treeitem" [attr.aria-selected]="((_value) && (item[indiceId] == _value[indiceId])) ? true : false">
                                    <span  [inner-template]="templateResultado || templateResultadoInterno" [item]="item"></span>
                                </li>
                            </span>
                            <li [style.display]="(valoresExibir.length > 0) ? 'none' : ''" class="select2-results__option select2-results__message" aria-live="assertive">
                                <span  [inner-template]="templateSemResultado || templateSemResultadoInterno" [item]="{pesquisa:valorPesquisado}"></span>
                            </li>
                        </span>
                        <li  [style.display]="(exibirMensagemCaracteresMinimo == false) ? 'none': ''" class="select2-results__option select2-results__message">
                            Digite {{minimoCaracteres}} ou mais caracteres para realizar a busca
                        </li>
                    </ul>
                </span>
            </span>
        </span>
        </div>



        <ng-template #templateSemResultadoInterno>
            Nenhum resultado encontrado
        </ng-template>
        <ng-template #templateResultadoInterno let-valor>
            {{valor[indiceNome]}}
        </ng-template>
        <ng-template #templateSelecionadoInterno let-valor>
        <span *ngIf="valor">
            {{valor[indiceNome]}}
        </span>
        </ng-template>

    `,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => SelectComponent),
                        multi: true
                    }
                ],
                host: {
                    '(document:click)': 'clickForaComponent($event)',
                    '(focus)': 'setFocus(true)',
                    '(blur)': 'setFocus(false)',
                },
            },] },
];
/** @nocollapse */
SelectComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
];
SelectComponent.propDecorators = {
    "name": [{ type: Input },],
    "classe": [{ type: Input },],
    "placeholder": [{ type: Input },],
    "minimoCaracteres": [{ type: Input },],
    "templateResultado": [{ type: Input },],
    "templateSelecionado": [{ type: Input },],
    "templateSemResultado": [{ type: Input },],
    "indiceId": [{ type: Input },],
    "indiceNome": [{ type: Input },],
    "disabled": [{ type: Input },],
    "valores": [{ type: Input },],
    "onSelecionarItem": [{ type: Output },],
    "onBuscar": [{ type: Output },],
    "onApagar": [{ type: Output },],
    "onRemoverItem": [{ type: Output },],
    "onAbrir": [{ type: Output },],
    "onFechar": [{ type: Output },],
    "onLimpar": [{ type: Output },],
    "campoBusca": [{ type: ViewChild, args: ['campoBusca',] },],
    "value": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TagComponent extends Select {
    /**
     * @param {?} elementRef
     * @param {?} zone
     */
    constructor(elementRef, zone) {
        super(elementRef, zone);
        this.elementRef = elementRef;
        this.zone = zone;
        this.data = [];
        this._valores = [];
        this.name = '';
        this.classe = '';
        this.placeholder = 'Selecione';
        this.disabled = false;
        this.change = new EventEmitter();
        this.onSelecionarItem = new EventEmitter();
        this.onBuscar = new EventEmitter();
        this.onApagar = new EventEmitter();
        this.onRemoverItem = new EventEmitter();
        this.onAbrir = new EventEmitter();
        this.onFechar = new EventEmitter();
        this.onLimpar = new EventEmitter();
    }
    /**
     * @param {?} valor
     * @return {?}
     */
    set valores(valor) {
        this._valores = valor;
        this.valoresExibir = valor;
    }
    /**
     * @return {?}
     */
    abrir() {
        if (this.disabled) {
            return false;
        }
        if (this.aberto) {
            this.fechar();
        }
        else {
            this.aberto = true;
            if (!this._value) {
                this._value = [];
            }
            this.valorPesquisado = '';
            this.campoBusca.nativeElement.focus();
            this.setFocus(true);
            this.onAbrir.emit(true);
            this.buscar();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    keyup(event) {
        event.preventDefault();
        let /** @type {?} */ valor = event.target.value;
        //enter
        if (event.keyCode == 13) {
            this.selecionar(valor, false);
            this.valorPesquisado = '';
            this.buscar();
            return;
        }
        //apagar
        if (event.keyCode == 8) {
            this.remove(this.data[this.data.length - 1], event);
            this.valorPesquisado = '';
            this.buscar();
            event.preventDefault();
            return;
        }
        if (this.valorPesquisado == valor) {
            return null;
        }
        this.valoresExibir = [];
        this.valorPesquisado = valor;
        this.buscar();
    }
    /**
     * @param {?} item
     * @param {?=} fecharCampo
     * @return {?}
     */
    selecionar(item, fecharCampo = true) {
        item = item.trim();
        if (item == '') {
            return false;
        }
        //verifica se ja foi adicionado
        let /** @type {?} */ jaExiste = this.data.findIndex(procurar => procurar == item);
        if (jaExiste === -1) {
            this.data.push(item);
        }
        this.updateValue(this.data);
        this.onSelecionarItem.emit(item);
        if (fecharCampo) {
            this.fechar();
        }
        //apaga campo de busca
        this.campoBusca.nativeElement.value = '';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    limpar(event) {
        event.stopPropagation();
        this.data = [];
        this.updateValue(null);
        this.onLimpar.emit(true);
        this.aberto = false;
        this.abrir();
    }
    /**
     * @param {?} item
     * @param {?} event
     * @return {?}
     */
    remove(item, event) {
        event.stopPropagation();
        // this.fechar();
        // let itens
        let /** @type {?} */ posicao = this.data.findIndex(procurar => procurar == item);
        if (posicao !== -1) {
            this.data.splice(posicao, 1);
        }
        this.onRemoverItem.emit(item);
        // if(this.data.length == 0){
        //     this.aberto = false;
        //     this.abrir();
        // }
        this.updateValue(this.data);
    }
    /**
     * @return {?}
     */
    buscar() {
        if (this.validaCaracteresMinimo() == false) {
            return false;
        }
        this.onBuscar.emit(this.valorPesquisado);
        this.valoresExibir = this._valores.filter(item => {
            let /** @type {?} */ pesquisa = item.toString().toLocaleLowerCase().indexOf(this.valorPesquisado.toString().toLocaleLowerCase());
            //verifica se o item da lista original ja n esta add na lista data
            let /** @type {?} */ jaFoiAdicionado = this.data.findIndex(procurar => procurar == item);
            if (pesquisa !== -1 && jaFoiAdicionado === -1) {
                return item;
            }
        });
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    ;
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = (v);
            this._onChangeCallback(v);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (!Array.isArray(value)) {
            value = [];
        }
        this.data = value;
        this._value = value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.zone.run(() => {
            this._value = value;
            this.onChange(value);
            this._onTouchedCallback();
            this.change.emit(value);
        });
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onChange(_) { }
    /**
     * @return {?}
     */
    onTouched() { }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) { this.onChange = fn; }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) { this.onTouched = fn; }
    /**
     * @param {?} _
     * @return {?}
     */
    _onChangeCallback(_) { }
    /**
     * @return {?}
     */
    _onTouchedCallback() { }
}
TagComponent.decorators = [
    { type: Component, args: [{
                selector: 'select2-tag',
                template: `
        <!--select simples-->
        <span [ngClass]="getClassGeral()" dir="ltr" (click)="abrir()">
        <span class="selection">
            <span class="select2-selection select2-selection--multiple" role="combobox" aria-haspopup="true" aria-expanded="false" tabindex="-1">
            <ul class="select2-selection__rendered">
                <span [hidden]="data.length == 0" >
                    <span [inner-template]="templateSelecionado || templateSelecionadoInterno" [item]="this"></span>
                </span>
                <li class="select2-search select2-search--inline" style="float:none">
                    <input (keyup)="keyup($event)" (keydown.enter)="$event.preventDefault()" #campoBusca class="select2-search__field" type="search" tabindex="0" autocomplete="off" autocorrect="off" autocapitalize="off" spellcheck="false" role="textbox" [placeholder]="(data.length == 0) ? placeholder : ''">
                </li>
                <li class="select2-selection__arrow" role="presentation"><b role="presentation"></b></li>
            </ul>
        </span>
        </span>
        <span class="dropdown-wrapper" aria-hidden="true"></span>
    </span>

        <!--quando clicar campo-->
        <div [hidden]="!aberto">
        <span class="select2-container select2-container--bootstrap select2-container--open">
            <span class="select2-dropdown select2-dropdown--below" style="z-index: 999999" dir="ltr" >
                <span class="select2-results">
                    <ul class="select2-results__options" role="tree"  aria-expanded="true" aria-hidden="false">
                        <span [style.display]="(exibirMensagemCaracteresMinimo) ? 'none': ''">
                            <span [hidden]="valoresExibir.length == 0" >
                                <li *ngFor="let item of valoresExibir" (click)="selecionar(item)" class="select2-results__option" highlight="select2-results__option--highlighted" role="treeitem" [attr.aria-selected]="((_value) && (item == _value)) ? true : false">
                                    <span  [inner-template]="templateResultado || templateResultadoInterno" [item]="item"></span>
                                </li>
                            </span>
                            <li [style.display]="(valoresExibir.length > 0) ? 'none' : ''"  class="select2-results__option select2-results__message" aria-live="assertive">
                                <span  [inner-template]="templateSemResultado || templateSemResultadoInterno" [item]="{pesquisa:valorPesquisado}"></span>
                            </li>
                        </span>
                        <li [style.display]="(exibirMensagemCaracteresMinimo == false) ? 'none': ''" class="select2-results__option select2-results__message">
                            Digite {{minimoCaracteres}} ou mais caracteres para realizar a busca
                        </li>
                    </ul>
                </span>
            </span>
        </span>
        </div>



        <ng-template #templateSemResultadoInterno>
            Nenhum resultado encontrado
        </ng-template>
        <ng-template #templateResultadoInterno let-valor>
            {{valor}}
        </ng-template>
        <ng-template #templateSelecionadoInterno let-valor>
        <span *ngIf="valor">
            <li *ngFor="let item of valor.data" class="select2-selection__choice">
                <span class="select2-selection__choice__remove" role="presentation" (click)="valor.remove(item, $event)">×</span>
                {{item}}
            </li>
        </span>
        </ng-template>
    `,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => TagComponent),
                        multi: true
                    }
                ],
                host: {
                    '(document:click)': 'clickForaComponent($event)',
                    '(focus)': 'setFocus(true)',
                    '(blur)': 'setFocus(false)',
                },
            },] },
];
/** @nocollapse */
TagComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
];
TagComponent.propDecorators = {
    "name": [{ type: Input },],
    "classe": [{ type: Input },],
    "placeholder": [{ type: Input },],
    "templateResultado": [{ type: Input },],
    "templateSelecionado": [{ type: Input },],
    "templateSemResultado": [{ type: Input },],
    "disabled": [{ type: Input },],
    "valores": [{ type: Input },],
    "change": [{ type: Output },],
    "onSelecionarItem": [{ type: Output },],
    "onBuscar": [{ type: Output },],
    "onApagar": [{ type: Output },],
    "onRemoverItem": [{ type: Output },],
    "onAbrir": [{ type: Output },],
    "onFechar": [{ type: Output },],
    "onLimpar": [{ type: Output },],
    "campoBusca": [{ type: ViewChild, args: ['campoBusca',] },],
    "value": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Ng2Select2BootstrapModule {
}
Ng2Select2BootstrapModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                ],
                exports: [
                    InnerTemplateDirective,
                    SelectComponent,
                    MultipleComponent,
                    RequestComponent,
                    RequestMultipleComponent,
                    FormsModule,
                    TagComponent,
                    HighlightDirective,
                ],
                declarations: [
                    InfiniteScrollDirective,
                    InnerTemplateDirective,
                    SelectComponent,
                    MultipleComponent,
                    RequestComponent,
                    HighlightDirective,
                    RequestMultipleComponent,
                    TagComponent,
                    HiddenDirective,
                ],
                providers: [SelectService]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { SelectService, HiddenDirective, HighlightDirective, InfiniteScrollDirective, InnerTemplateDirective, MultipleComponent, RequestComponent, RequestMultipleComponent, SelectComponent, TagComponent, Select, Ng2Select2BootstrapModule };
//# sourceMappingURL=ng2-select2-bootstrap.js.map
